
/** \file
Tacho - Zeigt in Abhngigkeit einer Pulsfrequenz eine
Geschwindigkeit, Strecke, Zeit seit Start und Durchschnittsgeschwindigkeit seit Start an.

Die Strecke je Puls ist fest vorgegeben (METER_JE_TIC_BASE),
lt sich aber mittels Spannungswert (Poti) im Bereich von +/- 10% fein einstellen.

Die Einstellung gilt fr die Geschwindigkeit und die Strecke!



Realisiert mit PIC 16c76 (da wir mehr ROM brauchen als im 16c73B steckt)

               ROM used: 5154 (63%) Largest free fragment is 2048
               RAM used: 77 (44%) at main() level
                         126 (72%) worst case
               Stack:    4 worst case (3 in main + 1 for interrupts)







Pinbelegung:
\verbatim

     (ai=Analog In, do=digital out  dio=digital in/out  di=digital in)

     A0  (ai)  Light Sensor (Umgebungshelligkeit)
     A1  (ai)  Teilerspannung
     A2  (ai)  unused
     A3  (ai)  unused
     A4  (di)  Pulseingang (T0CKI)
     A5  (ai)  unused

     B0  (do) LCD D0
     B1  (do) LCD D1
     B2  (do) LCD D2
     B3  (do) LCD D3
     B4  (do) LCD D4
     B5  (do) LCD D5
     B6  (do) LCD D6
     B7  (do) LCD D7

     C0  (do)  LCD enable
     C1  (do)  LCD rs
     C2  (do)  LCD Backlight (PWM)
     C3  (di)  Feature-Jumper 1: 0= Bargraph 1=Bignum
     C4  (di)  Feature-Jumper 2: 0= Zeige kein AV (nur bei BigNum)     //     C4  (di)  Feature-Jumper 2: 0= Zeige Tachopuls (nur bei BigNum)
     C5  (di)  Feature-Jumper 3: 0= volle Helligkeit 1=LDR-gefhrte Helligkeit
     C6  (di)  Feature-Jumper 4: 0= Zeige auch Teilerverhaeltnis (nicht bei BigNum) 1= Zeige nur Geschwingigkeit
     C7  (di)  Feature-Jumper 5: 0= Unterdrcke 0 Km/h  1= Zeige 0 Km/h (Nur bei BigNum)

\endverbatim
*/


// ----------------------------------------------------------------------------
//  PIC Programmer
// ----------------------------------------------------------------------------
//#device PIC16C73B
//#device PIC16C73
#device PIC16C76


//#include <16C73b.H>
#include <16C76.H>
//#include <stdlib.h>
#include <math.h>

// HighSpeed,NoProtection,WatchdogTimer,PowerUpTimer,BrownoutDetection
#fuses HS,NOPROTECT,WDT,PUT,BROWNOUT

//Hinweis: Wenn #fuses weiter oben stehen, funktionieren sie nicht!
// ----------------------------------------------------------------------------
//  Constants and compiler options
// ----------------------------------------------------------------------------

//#define METER_JE_TIC 0.3952
#define METER_JE_TIC_BASE 0.3952


#define CLOCK_SPEED 4000000  // 4MHz
#define T0_RESET_VAL 0xff
#define T1_START (T1_INTERNAL | T1_DIV_BY_8)
// need to know the clock speed for delay
#use delay(clock=CLOCK_SPEED)


//Umrechnungsfaktor gemaess ClockSpeed
// 3.6 km/h je m/s  Clockspeed/4 an T1  /8: T1_DIV_BY_8   /10:fr Justierung wird spter *10
//#define V_FAKTOR (METER_JE_TIC * 3.6 * CLOCK_SPEED/4/8 / 10.0) - oder bereits teilweise ausgerechnet:
//#define V_FAKTOR (METER_JE_TIC_BASE * 45000)


// we never change the port direction, so we use fast mode I/O.
#use FAST_IO(A)
#use FAST_IO(B)
#use FAST_IO(C)


// ----------------------------------------------------------------------------
//  Port maps
// ----------------------------------------------------------------------------

struct portAMap {
    boolean licht;
    boolean poti;
    boolean unused2;
    boolean unused3;
    boolean unused4;
    boolean unused5;
  }portA;

struct portBMap {
    int data : 8;
  }portB;

struct portCMap {
    boolean lcdEnable;
    boolean lcdRS;
    boolean unused1;
    boolean feature1;
    boolean feature2;
    boolean feature3;
    boolean feature4;
    boolean feature5;
  }portC;

#byte portA = 5                        // This puts the entire structure
                                       // on to port A (at address 5)
#byte portB = 6                        // This puts the entire structure
                                       // on to port B (at address 6)
#byte portC = 7                        // This puts the entire structure
                                       // on to port C (at address 7)

#byte adcResultRegister = 0x1e    //AD-Wandler Ergebnis
#byte adcon0Register    = 0x1f    //AD-Wandler Kontrollregister

STRUCT portAMap const TRIS_A = {1,1,1,1,1,1};     // All IN
STRUCT portBMap const TRIS_B = {0};               // all OUT
STRUCT portCMap const TRIS_C = {0,0,0,1,1,1,1,1}; // OUT for display, all others IN
/*
byte const userCharDef[64] = { //8Zeichen mit je 8 Bytes. Bit0 ist Pixel rechts (Bit 5-7:don't care)
                               //1.Byte ist obere Zeile, letzte Byte je Zeichen ist egal (Da im Display dafr keine Pixel vorhanden sind)
    0,  0,  0,  0,  0,  0,  0,0, //0 nix
  135,135,135,135,135,135,135,0, //1 Balken rechts
  156,156,156,156,156,156,156,0, //2 Balken links
  159,  0,  0,  0,  0,159,159,0, //3 Balken unten+Strich oben
  159,159,  0,  0,  0,  0,159,0, //4 Balken oben+Strich unten
    0,  0,  0,  0,  0,159,159,0, //5 Balken unten
  159,159,  0,  0,  0,  0,  0,0, //6 Balken oben
  159,159,  0,  0,  0,159,159,0  //7 Balken oben und unten
};
byte const ziffer[10][6] ={
  {1,6,2,1,5,2}, //0
  {0,0,2,0,0,2}, //1
  {0,4,2,1,3,0}, //2
  {0,4,2,0,3,2}, //3
  {1,0,2,0,6,2}, //4
  {1,4,0,0,3,2}, //5
  {1,6,0,1,7,2}, //6
  {0,6,2,0,0,2}, //7
  {1,7,2,1,5,2}, //8
  {1,7,2,0,5,2}  //9
};
*/
byte const userCharDef[64] = { //8Zeichen mit je 8 Bytes. Bit0 ist Pixel rechts (Bit 5-7:don't care)
                               //1.Byte ist obere Zeile, letzte Byte je Zeichen ist egal (Da im Display dafr keine Pixel vorhanden sind)
  255,255,255,255,255,255,255,0, //0 Block
  135,135,135,135,135,135,135,0, //1 Balken rechts
  156,156,156,156,156,156,156,0, //2 Balken links
  159,  0,  0,  0,  0,159,159,0, //3 Balken unten+Strich oben
  159,159,  0,  0,  0,  0,159,0, //4 Balken oben+Strich unten
    0,  0,  0,  0,  0,159,159,0, //5 Balken unten
  159,159,  0,  0,  0,  0,  0,0, //6 Balken oben
  159,159,  0,  0,  0,159,159,0  //7 Balken oben und unten
};
byte const ziffer[10][6] ={
  //   obere  Zeile  untere      Ziffer
  {  0,  6,  0,    0,  5,  0}, //0
  { 32, 32,  0,   32, 32,  0}, //1
  {  6,  6,  0,    0,  7,  5}, //2
  {  6,  4,  0,    5,  3,  0}, //3
  {  0, 32,  0,   32,  6,  0}, //4
  {  0,  7,  6,    5,  5,  0}, //5
  {  0,  6,  6,    0,  7,  0}, //6
  {  0,  6,  0,   32, 32,  0}, //7
  {  0,  7,  0,    0,  5,  0}, //8
  {  0,  7,  0,    5,  5,  0}  //9
};
// ----------------------------------------------------------------------------
//  Global variables
// ----------------------------------------------------------------------------

struct globVar{
  byte rsLine;
  //long speed;
  int overflows;
  long timeticksL;  //long ist unsigned!
  int timeticksH;
  int licht, poti, potiMeasured;
  byte newValue;
  byte hr,min,sec;
  long kmTic;
  float vFaktor10;
  float potiFaktor;
  float meterJeTicCal; //Mittels Poti kalibriert
}glob;

// ----------------------------------------------------------------------------
//! Prozessor neu starten

void softReset(void){
#asm
  goto 0
#endasm
}


// ----------------------------------------------------------------------------
//   LCD functions
// ----------------------------------------------------------------------------



// ---------------------------------------------------------------------------
/*! Send byte to display.
    Depending on the value of glob.rsLine the byte is written to display(glob.rsLine=1) or register (glob.rsLine=0)

   \param b is the byte to be send.
   \note the BUSY-Flag isn't ckecked (since we can't read from display with R/W hard-wired to GND)
*/// -------------------------------------------------------------------------
void lcdSendByte(byte b ) {
      restart_wdt();
      portC.lcdRS = glob.rsLine;
      delay_cycles(1);  //t_AS: 40ns min  #BATRON, page 57
      portC.lcdEnable = 1;
      //delay_cycles(4);  // not necessary

      portB.data = b;
      delay_cycles(1); //t_DSW: 80ns min.

      portC.lcdEnable = 0;
      delay_us(1); //t_H: 10ns min

      delay_us(1);  //t_CYC 500ns min

      // CAUTION: we do not check the BUSY-Flag (since we can't read from display)
      // CAUTION: so for HOME and CLEAR commands the user has to add 2ms delay!

      //500ns seems too short for t_CYC. Display somtimes didn't catch a char.
      //so we delay a bit more...
      delay_us(100);
}


// ---------------------------------------------------------------------------
/*! Clear LCD and set cursor home
   (HD 44780 based displays only)
*/// -------------------------------------------------------------------------

void clearLcd(void) {
  glob.rsLine=0;
  lcdSendByte(0x01);
  glob.rsLine=1;
  delay_ms(2);
  restart_wdt();
}
// ---------------------------------------------------------------------------
/*! Set cursor home
   (HD 44780 based displays only)
*/// -------------------------------------------------------------------------
void home1Lcd(void) {
    glob.rsLine=0;
    lcdSendByte(0x02);
    glob.rsLine=1;
    delay_ms(2);
    restart_wdt();
}
// ---------------------------------------------------------------------------
/*! Set cursor at start of Line 2
   (HD 44780 based displays only)
*/// -------------------------------------------------------------------------
void home2Lcd(void) {
    glob.rsLine=0;
    lcdSendByte(192);
    glob.rsLine=1;
    delay_ms(2);
    restart_wdt();
}
// ---------------------------------------------------------------------------
/*! Define user characters
   (HD 44780 based displays only)
*/// -------------------------------------------------------------------------

void defineUserChar(void) {
  int i,n;

  glob.rsLine=0;
  lcdSendByte(64); //Address first address in CharacterGenerator-Ram
  glob.rsLine=1;
  delay_us(100);
  for (n=0; n<64; n++){ //8 characters * 8 bytes for each character
    lcdSendByte(userCharDef[n]);
  }
//  delay_ms(2);
  home1Lcd();
//  delay_ms(5);
  clearLcd(); //Necessary to switch from CG-Ram to Display-RAM
  //now char 0x00...0x07 are user-defined
}



// ---------------------------------------------------------------------------
/*! Show SW-Version on LCD (at startup)
*/// -------------------------------------------------------------------------
void lcdWelcomeMessage(void){
    clearLcd();       //123456789 123456789
    set_pwm1_duty(128);
    printf(lcdSendByte,"   Dr. Schroeder   ");
}

// ---------------------------------------------------------------------------
/*! Initialize display (2 Lines, 8 Bit, no shift, increment)
   (HD 44780 based displays only)
   Writes "LCD Ready" on display after initialization
*/// -------------------------------------------------------------------------
void lcdInit(void) {

    portC.lcdRs = 0;
    portC.lcdEnable = 0;
    glob.rsLine=0;

    restart_wdt();
    delay_ms(15);
    restart_wdt();

    lcdSendByte(0x30); // DB5=1 DB4=1        # see BATRON-Specification for HD44780, page 58
    delay_ms(5);      // wait more than 4.1ms

    lcdSendByte(0x30); // DB5=1 DB4=1
    delay_ms(1);       // wait more than 100us

    lcdSendByte(0x30);
    delay_ms(2);	   //just to be sure...
    restart_wdt();

    lcdSendByte(0x38); //2 Lines, Font 0 (5x7 dot)  N=1 F=0
    lcdSendByte(0x0c); // Display on
    delay_us(2);

    lcdSendByte(0x01); // Display clear
    delay_ms(3);

    lcdSendByte(0x06); // Entry mode; no shift
    delay_ms(2);
    glob.rsLine=1;
    restart_wdt();
}

// ---------------------------------------------------------------------------
#inline long calcSpeed(void){
//  float v,inv;
//  long speed;
//  speed=0;
  //Geschwindigkeit aus Pulsabstand berechnen
/*
  inv=glob.timeticksH;
  inv*=65536;
  inv+=glob.timeticksL;
  inv=(float)glob.timeticksH*65536.0+(float)glob.timeticksL;
*/
  if (glob.timeticksL || glob.timeticksH){  //Schutz vor "div zero"
    if (glob.timeticksH > 6){
      return 0;
    }else{
//      v=glob.vFaktor10 / inv;
//      v=glob.vFaktor10 / ((float)glob.timeticksH*65536.0+(float)glob.timeticksL);
      return floor(glob.vFaktor10 / ((float)glob.timeticksH*65536.0+(float)glob.timeticksL));
    }
  }else{
    return 255;
  }
  return 0;
}
#if 1
// ---------------------------------------------------------------------------
void werteAnzeigen(long v){
  byte h,z,e,tmp;

  h=v/100;
  tmp=v % 100;
  z=tmp/10;
  e=tmp%10;


  home1Lcd();
  if (!portC.feature4){
    printf(lcdSendByte,"L%3u",glob.licht);
  }else{
    lcdSendByte(' ');
    lcdSendByte(' ');
    lcdSendByte(' ');
    lcdSendByte(' ');
  }
  lcdSendByte(' ');
  lcdSendByte(' ');
  lcdSendByte(' ');
  lcdSendByte(' ');
  if (h>0){
    lcdSendByte(48+ h);
  }else{
    lcdSendByte(' ');
  }
  if ((z>0) || (h>0)){
    lcdSendByte(48+ z);
  }else{
    lcdSendByte(' ');
  }
  lcdSendByte(48+ e);
  lcdSendByte(' ');

  if (!portC.feature4){
    lcdSendByte(' ');
    lcdSendByte(' ');
    lcdSendByte(' ');
    glob.poti = glob.potiMeasured;
    printf(lcdSendByte,"P%u",glob.poti);
    lcdSendByte(' ');
  }
/*
  if (portC.debug){
    home2Lcd();

    printf(lcdSendByte,"%d %lu L%u P%u ",glob.timeticksH,glob.timeticksL,
      glob.licht,glob.poti);
  }
*/
}
#endif
#if 0
// ---------------------------------------------------------------------------
#inline tachopulsAnzeigen(void){
  home2Lcd();
  if (portA.unused4){
    lcdSendByte(' ');
  }else{
    lcdSendByte('*');
  }
}
#endif
// ---------------------------------------------------------------------------
void drawBigNum(int num, int hr, int m, int s, long dist, int vmittel){
  byte h,z,e,tmp;

  //------Schreibe 1.Zeile--------
  h=num/100;
  tmp=num % 100;
  z=tmp/10;
  e=tmp%10;
  home1Lcd();
  if (h>0){
    lcdSendByte(ziffer[h][0]);
    lcdSendByte(ziffer[h][1]);
    lcdSendByte(ziffer[h][2]);
  }else{
    lcdSendByte(' ');
    lcdSendByte(' ');
    lcdSendByte(' ');
  }
  lcdSendByte(' ');
  if ((z>0) || (h>0)){
    lcdSendByte(ziffer[z][0]);
    lcdSendByte(ziffer[z][1]);
    lcdSendByte(ziffer[z][2]);
  }else{
    lcdSendByte(' ');
    lcdSendByte(' ');
    lcdSendByte(' ');
  }

  lcdSendByte(' ');

  if ((e>0) || (z>0) || (h>0) || portC.feature5){
    lcdSendByte(ziffer[e][0]);
    lcdSendByte(ziffer[e][1]);
    lcdSendByte(ziffer[e][2]);
  }else{
    lcdSendByte(' ');
    lcdSendByte(' ');
    lcdSendByte(' ');
  }
  lcdSendByte(' ');
//  lcdSendByte(' ');
  if (dist>999){
    printf(lcdSendByte,"%3lu.%03lum",dist/1000,dist%1000);
  }else{
    lcdSendByte(' ');
    lcdSendByte(' ');
    lcdSendByte(' ');
    lcdSendByte(' ');
    printf(lcdSendByte,"%3lum",dist);
  }
  lcdSendByte(' ');
  lcdSendByte(' ');

  //------Schreibe 2.Zeile------
  home2Lcd();
  if (h>0){
    lcdSendByte(ziffer[h][3]);
    lcdSendByte(ziffer[h][4]);
    lcdSendByte(ziffer[h][5]);
  }else{
    lcdSendByte(' ');
    lcdSendByte(' ');
    lcdSendByte(' ');
  }
  lcdSendByte(' ');
  if ((z>0) || (h>0)){
    lcdSendByte(ziffer[z][3]);
    lcdSendByte(ziffer[z][4]);
    lcdSendByte(ziffer[z][5]);
  }else{
    lcdSendByte(' ');
    lcdSendByte(' ');
    lcdSendByte(' ');
  }
  lcdSendByte(' ');
  if ((e>0) || (z>0) || (h>0) || portC.feature5){
    lcdSendByte(ziffer[e][3]);
    lcdSendByte(ziffer[e][4]);
    lcdSendByte(ziffer[e][5]);
  }else{
    lcdSendByte(' ');
    lcdSendByte(' ');
    lcdSendByte(' ');
  }
  lcdSendByte(' ');
  //Merkwrdigerweise wird das Display beim Anzeigen von AV kontrastrmer!?!
  //Evtl das printf der LONG Var? - n
  //Evtl das printf mit abschliessenden Spaces? - n
  //Evtl das printf ber den rechten Rand? - n
  //--war ein HW-Defekt im Display

  if (((s & 3)>1) && portC.feature2){  //2 sec Zeit, 2 sec Durchschnittsgeschwindigkeit
//    printf(lcdSendByte,"%4lu AV ",vmittel);
    lcdSendByte(' ');
    lcdSendByte(' ');
    lcdSendByte(' ');
    h=vmittel/100;
    tmp=vmittel % 100;
    z=tmp/10;
    e=tmp%10;
    if (h>0){
      lcdSendByte(h+48);
    }else{
      lcdSendByte(' ');
    }
    if ((z>0) || (h>0)){
      lcdSendByte(z+48);
    }else{
      lcdSendByte(' ');
    }
    lcdSendByte(e+48);
    lcdSendByte(' ');
    lcdSendByte(' ');
  }else{
    if (hr>0){
      printf(lcdSendByte,"%2u:%02u:%02u",hr,m,s);
    }else{
      printf(lcdSendByte,"   %2u:%02u",m,s);
    }
  }
}

#if 0
// ---------------------------------------------------------------------------
void drawBar(int length){
  char bar[21]="----+----O----+-****";
  int i;
  home2Lcd();
  for (i=0; i<20; i++){
    if (i<length){
      lcdSendByte(bar[i]);
    }else{
      lcdSendByte(' ');
    }
  }
}

// ---------------------------------------------------------------------------
#inline void balkenanzeige(long v){
  if (v < 20){
    drawBar(v);
  }else
  if (v < 40){
    drawBar(v-20);
  }else
  if (v < 60){
    drawBar(v-40);
  }else
  if (v < 80){
    drawBar(v-60);
  }else
  if (v < 120){
    drawBar((v-80)/2);
  }else
  if (v < 160){
    drawBar((v-120)/2);
  }else{
    drawBar(0);
  }
}
#endif
// ----------------------------------------------------------------------------
//  ADC
// ----------------------------------------------------------------------------

void getAdc(void){
  //Liest die Analogwerte ein und aktualisiert die globalen
  //Variablen licht und poti.
  static long sum1=0;
  static long sum2=0;
  static byte adChannel=0;
  static byte n=0;

  switch (adChannel){

    case 0:  sum1+=(long)adcResultRegister; //Read from ADC result register
      adChannel=1;
      break;

    case 1: sum2+=(long)adcResultRegister;
      adChannel=0;
      n++;
      break;

    default:
      adChannel=0;
      break;
  }

  if (n>3){ //4 mal gemessen. Sum: 0..255*4, also 0..1020
    n=0;
    glob.licht=(int)(sum1/6);  //0..170
    glob.potiMeasured=1+(int)(sum2/4); //0..255
    sum1=0;
    sum2=0;
  }
  set_adc_channel(adChannel);
  delay_us(100); //Damit sich der Sample/Hold Kondensator anpassen kann

  //neue AD-Wandlung starten (darf nicht direkt nach dem Auslesen passieren)
  bit_set(adcon0Register,2); // ADC GO

}
// ---------------------------------------------------------------------------
/*!   Interrupt service functions
*/// -------------------------------------------------------------------------

#int_default
void doNothing(void){
}


// ---------------------------------------------------------------------------
#int_rtcc
void t0_isr(void){
  //Timer 0 wieder scharf machen
  set_timer0(T0_RESET_VAL);
  //Timer 1 auslesen
  glob.timeticksL = get_timer1();
  glob.timeticksH = glob.overflows;
  glob.newValue=1;
  //und zurcksetzen
  set_timer1(0);
  glob.overflows=0;
  //neu starten (falls er nach berlauf in t1_isr gestoppt war)
  setup_timer_1(T1_START);
  glob.kmTic++;
}

// ---------------------------------------------------------------------------
#int_timer1
void t1_isr(void){
  //Timer 1 berlauf
  glob.overflows++;
  if (glob.overflows > 1){
    //: Halte ihn an.
    glob.timeticksL = 65535;
    glob.timeticksH = 8;
    setup_timer_1(T1_DISABLED);
    glob.newValue=1;
  }
}

// ---------------------------------------------------------------------------
#int_timer2
void t2_isr(void){
  static long tic=0;
  //Timer 2 (alle 1.6ms )
  tic++;
  if (625==tic){  //1.6ms*625 = 1000ms
    tic=0;
    glob.sec++;
  }
}
// ---------------------------------------------------------------------------
#inline void setBrightness(void){
int brightness;
  if (portC.feature3){
    if (glob.licht>110){
      brightness=18;
    }else{
      brightness=128 - glob.licht;  //licht:0 wenn Hell, 128 wenn Dunkel
    }
    if (brightness>127){
      brightness=128;
    }
  }else{
    brightness=128;
  }
  set_pwm1_duty(brightness);
}
// ---------------------------------------------------------------------------
/*! Preset all global variables
*/// -------------------------------------------------------------------------
#inline void presetGlobalVar(void){
  glob.overflows=0;
  glob.kmTic=0;
  glob.meterJeTicCal=METER_JE_TIC_BASE;
  glob.vFaktor10 = glob.meterJeTicCal * 450000;  //450000: Siehe unten
}
// -----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
//  MAIN
// ----------------------------------------------------------------------------
int main(void){
int i,j;
int n;
int potiBisher=0;
//int demo, democnt;
long vsum;
long speed;
long dist;
int  vmit;
//float strecke100km;
float gesamtzeit;
  // keine interrupts
  disable_interrupts(GLOBAL);

  // Setup ports
  set_tris_A(TRIS_A);
  set_tris_B(TRIS_B);
  set_tris_C(TRIS_C);

  // Setup AnalogDigitalConverter
  setup_adc(ADC_CLOCK_INTERNAL);
  setup_adc_ports(RA0_RA1_RA3_ANALOG);  //RA0 is analog input; internal reference;
  set_adc_channel(0); //Kanal, den wir lesen


  /*Timer 0: Pulserfassung und Interruptauslsung
    Der Timer wird mit 0xff vorbesetzt, so dass der nchste Puls einen Overflow
    erzeugt und damit ein INT ausgelst wird.

    Timer1: Prescaler/8 ==> 125KHz
    Mit die Zeit zwischen 2 Pulsen. Wird bei Timer0-INT gestoppt

    Timer2: PWM fr Hintergrundbeleuchtung

  */

  setup_counters(RTCC_EXT_L_TO_H | 16, WDT_2304MS);  //16: Kein Prescaler
  setup_timer_1(T1_START);  //125KHz



  //Interrupts fr Timer 0 und 1

  disable_interrupts(INT_RDA);  //Serial interface
  enable_interrupts(INT_TIMER2);

  enable_interrupts(INT_RTCC);
  enable_interrupts(INT_TIMER1);

  // Setup PWM
  setup_ccp1(CCP_PWM);   // Configure CCP1 as a PWM
  set_pwm1_duty(0);  //0..128

//  setup_timer_2(T2_DIV_BY_16, 127, 1);
  // The cycle time will be (1/clock)*4*t2div*(period+1)
  // In this program clock=4000000 and period=127
  //  cycle time = (1/4000000)*4*16*1*128 =^ 488.28125 Hz

  setup_timer_2(T2_DIV_BY_16, 99, 1);
  // The cycle time will be (1/clock)*4*t2div*(period+1)
  // In this program clock=4000000 and period=127
  //  cycle time = (1/4000000)*4*16*1*100 =1.6ms = 625.000 Hz

  //Warte 300ms, bis Display bereit ist
  for (i=0; i<30; i++){
    restart_wdt();
    delay_ms(10);
  }
  lcdInit();

  presetGlobalVar();
  enable_interrupts(GLOBAL);
  set_timer0(T0_RESET_VAL);

  lcdWelcomeMessage();

  //Welcome stehen lassen
  //und schon mal messen bevor etwas angezeigt wird
  for (i=0; i<60; i++){
    getAdc();
    delay_ms(10);
    restart_wdt();
  }
  //Dieser Wert wird jetzt nicht mehr verndert
  glob.poti = glob.potiMeasured;

  defineUserChar();  //macht auch  clearLcd();
  set_pwm1_duty(64);
  delay_ms(2);
  clearLcd();
                    //123456789 123456789
  printf(lcdSendByte,"       V 1.8       "); // Hier die aktuelle Versionsnummer eintragen

  home2Lcd();
  if (glob.poti<100){
    printf(lcdSendByte,"   -%d/1000 (%d)",glob.poti-100,glob.poti);
  }else{
    printf(lcdSendByte,"   +%d/1000 (%d)",glob.poti-100,glob.poti);
  }

  for (i=0; i<128; i++){
    set_pwm1_duty(i);
    getAdc();
    restart_wdt();
    delay_ms(10);
    restart_wdt();
    delay_ms(10);
  }

  n=1;
  vsum=0;

//  democnt=0;
//  demo=0;

  glob.hr=0;
  glob.min=0;
  glob.sec=0;
  dist=0;
//  strecke100km=0.0;
  gesamtzeit=1; //Darf nicht 0, weil es im Nenner steht
  clearLcd();

  //Hauptschleife
  while (1){
    set_timer0(T0_RESET_VAL); //Nur zur Sicherheit; Falls mal 1 interrupt verlorengeht...

    //Wenn am Poti gedreht wurde, neuen Faktor berechnen (Hysterese=2)
//    if (abs (glob.poti - potiBisher) > 1){
    if ((glob.poti > (potiBisher+1)) || (potiBisher > (glob.poti+1))){
      glob.meterJeTicCal=METER_JE_TIC_BASE * (10.0 + ((float)glob.poti-100.0) /100.0)/10.0; // +/- 10% Einstellbereich
      // 3.6 km/h je m/s  Clockspeed/4 an T1  /8: T1_DIV_BY_8
      //v_faktor =  (METER_JE_TIC * 3.6 * CLOCK_SPEED/4/8 ) - oder bereits teilweise ausgerechnet:

      glob.vFaktor10 = glob.meterJeTicCal * 450000.0;
//      glob.vFaktor10 = V_FAKTOR * (10 + ((float)glob.poti-100.0) /100); // +/- 10% Einstellbereich
      potiBisher = glob.poti;
    }

    //Echtzeituhr
    if (glob.sec>59){
      gesamtzeit+=60;
      glob.sec=0;
      glob.min++;
      if (glob.min>59){
        glob.min=0;
        glob.hr++;
        if (glob.hr>23){
          glob.hr=0;
        }
      }
    }
    //Meterzhler
    dist=floor(glob.meterJeTicCal * (float)glob.kmTic);
//    if (dist>99999){
//      glob.kmTic=0;
//      strecke100km+=100000.0;
//      dist=0;
//    }

    //Mittelwert Geschwindigkeit berechnen
    vsum/=n;
    n=1;
    //vsum bleibt stehen

    //Display aktualisieren
    if (portC.feature1){ //BigNum
      restart_wdt();
//      vmit=(int)(floor( (strecke100km + (float)dist) / (gesamtzeit+(float)glob.sec) *3.6 ));
      vmit=(int)(floor( (float)dist / (gesamtzeit+(float)glob.sec) *3.6 ));
      drawBigNum(vsum,glob.hr,glob.min,glob.sec,dist,vmit);
      restart_wdt();
    }else{               //Messwertanzeige
      werteAnzeigen(vsum);
    }

    //Warten (so dass etwa 2.5Hz Updatefrequenz bei rauskommen)
    for (i=0; i<19; i++){

      // beide Kanle 4 mal analog messen
      for (j=0; j<8; j++){
        getAdc();
        restart_wdt();
        delay_ms(2);
      }
      delay_ms(4);
      //Jetzt sind 20ms rum...

      if ((glob.newValue) || (8==glob.timeticksH)){ //Neuer Wert oder timeout (Auto steht)
        speed=calcSpeed();
        glob.newValue=0;
        vsum+=speed;
        n++;
      }
//      if (!portC.feature2){
//        tachopulsAnzeigen();
//      }
//      else
#if 0
      if (!portC.feature1){
        if (0==(i & 3)){        //Bargraph mit max. 50Hz/4 updaten
          balkenanzeige(vsum/n);
        }
      }
#endif
      if (0==(i & 3)){
        setBrightness(); //Helligkeit etwa alle 80 ms setzen
      }
    }
  }
  return 0;
}
// EOF

/*


  LC-DISPLAY: Writing to CG RAM.

  Writing to CG RAM is a lot like moving the cursor to a particular position on the display and
  displaying characters at that new location.
  The steps are:

  Reset RS and R/W pins of the LCD to prepare the LCD to accept instructions
  Set the CG RAM address by sending an instruction byte from 64 to 127 (locations 063 in CG RAM).
  Switch to DATA MODE by changing setting the RS pin
  Send bytes with the bit patterns for your symbol(s). The LCD controller automatically increments
  CG RAM addresses, just as it does cursor positions on the display.
  To leave CG RAM,switch to COMMAND MODE to set address counter to a valid display address
  (e.g. 128, 1st character of 1st line);
  the clear-screen instruction (byte 1); or the home instruction (byte 2).
  Now bytes are once again being written to the visible portion of the display.
  To see the custom character(s) you have defined, print ASCII codes 0 through 7.


*/